# Duden Korrektur Javascript Plugin

This is a TinyMCE Plugin for using DKS with Websites. It works with simple Textarea and Input Fields (for example in Web Forms) or with most WYSIWYG Editors. If you encounter Problems with a specific HTML Editor / Content Managment System, please [contact us](https://www.epc.de/) so we can add support for it.

### Requirements

- A DKS Server with either public API or an API User

### Installation Instructions

1. Unzip/Copy the "dkce" directory and put it somewhere on your webserver.

2. Add the DKCE Javascript Source on the page(s) you want to use the spellchecker.
```html
<script type="text/javascript" src="dkce/dkce.js"></script>
```

If you're using another path, please make sure to add the path also to the settings when initializing the plugin or the assets
cannot be found (Localization Files, Images).

### Write the initialization code

You then need to initialize the plugin to the elements of your page. You can attach it to *DIV*-Tags that have *contenteditable* set to *true*, to *Textarea*-Elements and to *Input*-Elements of type *text*.

The plugin itself scans all childs of the element you provide, so if you attach it to the body element of your page it will look for all
elements that are supported. You can also ommit the element completly and the Spellchecker tries to attach it to all elements where it makes sense (but this is not recommened).

Here is an example. This will initialize the Plugin to the Element with the ID *editor*.

```html
    <script>
        DKCE.init(document.querySelector('#editor'), { 
            apiEndpoint: 'http://127.0.0.1:8080/api',
            apiCredentials: {
                username: "duden",
                password: "duden"
            }
        });
    </script>
```

### Settings

There are a lot of different settings you can provide on initialisation. Here is a full list of all settings available.

- apiEndpoint
    - The api endpoint of the DKS Serer
    - Defaults to: http://127.0.0.1:8080/api
- apiCredentials
    - The api credentials required for the DKS Server. May be ommited if the DKS Api is not secured. You can either use username/password or an apiKey
        - username
        - password
        - apiKey
    - Defaults to: null
- configFile
    - Tries to load the given json file at startup and overwrites and update all other configuration settings.
    - Set it to null or empty string if you want to skip the HTTP Request to detect the file.
    - Defaults to: {path}/duden-config.json
- propertySets
    - Which property sets should the DKS use
    - Defaults to: ["base"]
- dictionaries
    - Which dictionaries sets should the DKS use
    - Defaults to: []
- detectManualHyphenation
    - If a user manually adds hyphenation, DKCE will auto detect it and replaces the '-' char with a soft hyphen
    - Defaults to: true
- language
    - Which language should the DKCE Plugin use
    - Defaults to the browser language of the user
- fallbackLanguage
    - Which fallback Language should the DKCE Plugin use, if the main language cannot be found. This is especially important when you ommit the language setting, and the language is detected automatically by the users browser settings.
    - Defaults to: "en"
- activated
    - Should the plugin be activated. If set to false, you can later activate it through the messaging api
    - Defaults to: true
- hideStatusPanel
    - Can be used to disable the Duden-Status-Panel at the top of the screen (which indicates network communication)
    - Defaults to: false
- hyphenation
    - Defaults to: true
- hyphenationStandard
    - Defaults to: null (Server Settings is used by default.)
- glossary
    - Defaults to: null (Server Settings is used by default.)
- hyphenationInStem
    - Defaults to: true
- hyphenationUnaest
    - Defaults to: null (Server Settings is used by default.)
- ignoreAttributes
    - When set it will ignore spellchecks for all child elements who has the given attribute in the HTML set (can be string or array of strings)
    - Defaults to: []
- ignoreClasses
    - When set it will ignore spellchecks for all child elements who has the given classes in the HTML set (can be string or array of strings)
    - Defaults to: []
- checkLanguages
    - When set it will ignore spellchecks for paragraphs that have a language attribute attached that are not defined here. If ommited all languages will be checked.
    - Defaults to: []
- correctionProposals
    - Defaults to: true
- singleWordMode
    - Defaults to: false
- styleForeign
    - Defaults to: null (Server Settings is used by default.)
- styleOld
    - Defaults to: null (Server Settings is used by default.)
- styleRegional
    - Defaults to: null (Server Settings is used by default.)
- styleColloquial
    - Defaults to: null (Server Settings is used by default.)
- styleFiller
    - Defaults to: null (Server Settings is used by default.)
- markupMode
    - Defaults to: "text"
- messageLanguage
    - Defaults to: null (Server Settings is used by default.)
- renderDelay
    - Stops the rendering of the highlights after a scroll event occurs for the given amount of time (in ms). Might be usefule for complex editors where highlights are lagging behind. A value of 0 means this feature is disabled and everything is rendered as fast as possible.
    - Defaults to: 0
- textLanguage
    - Defaults to: null (Server Settings is used by default.)
- messageLevel
    - Defaults to: null (Server Settings is used by default.)
- cssFiles
    - Array of CSS Files to use for DKCE. Please keep in mind that most of the css classes only make sense in markup mode
    - Defaults to: ['assets/css/dkce.css'] in current path
- colors
    - Definition which colors should be used (Please keep in mind, that in markUp Mode it may be overwritten from included cssFiles)
    - Defaults to: { spellingError: '#faa6ae', gramarError: '#abe89e', rejectedWord: '#abe89e', styleError: '#ffd7a8', glossaryInfo: '#ffff00' }
- spellcheckingLevel
    - Defaults to: null (Server Settings is used by default.)
- orthographyStandard
    - Defaults to: null (Server Settings is used by default.)

- openContextMenuOn
    - Defaults to 'mouseOver'
    - Possible Values are 'mouseOver', 'leftClick' and 'rightClick'

- debug
    - Only for internal usage, enables a debug bar
    - Defaults to: false
- isChromeExtension
    - Only for internal usage, is used by the chrome extension
    - Defaults to: false
- overlayMode
    - Can be used to disable overlay mode and instead use markup mode
    - Defaults to: true
- removeMarkupForSelection
    - If set to true (and instance is in MarkUp Mode) all markup code is removed from selections (and readded when selection is removed)
    - Defaults to: false
- path
    - Path to the dkce javascript source files. Is important so the plugin finds the assets like images and localization files.
    - Defaults to: "dkce"
- proposalDictionary
    - Can set the dictonary where proposals will be submitted to.
    - Defaults to: "Proposals"

### Initialisation Handlers

You can also add onSuccess and onFailure callbacks to the init function:

```html
    <script>
        var onSuccess = (dkce) =>  {
            // dkce is a javascript object that contains an api to
            // the dkce / spellcheckers instance(s)            
            
            // turn off Spellchecking
            dkce.activate(false);
            
            // returns true if spellchecking is active
            var isSpellcheckingActive = dkce.isActive(); 

            // Update DKCE Settings
            dkce.updateSettings({ spellcheckingLevel: 3 });

            // returns a js object with all current settings of the
            // dkce instance
            var settings = dkce.getCurrentSettings();

            // Display a DKCE Toast Message
            dkce.toast("Hello world!")

            // Propose a word to the DKS
            dkce.proposeWord("My word");
        };

        var onFailure = (error) => {
            console.log(error);
        };

        DKCE.init(document.querySelector('#editor'), settings, onSuccess, onFailure);
    </script>
```

### Communication with the Plugin

You can send messages to the Plugin, for example to activate it or to update the user settings. This is done through the Browsers Message System.

#### Activate the plugin

```js
window.postMessage({
    'subject': 'activate', 
    'activate': true
}, window.location.href);
```
#### Deactivate the plugin

```js
window.postMessage({
    'subject': 'activate', 
    'activate': true
}, window.location.href);
```

#### Update the settings

```js
window.postMessage({
    'subject': 'update_settings', 
    'settings': {
        /* Whatever settings you want to update */
    }
}, window.location.href);
```

#### Listen to content updates

Whenever the content of the element has updated the DKCE is attached to, a *content_changed* event is sent back via the messaging api. You can listen to it like this:

```js
window.addEventListener("message", (event) => { 
    if (event.data === 'content_changed') {
        /* Do something */
    }
};
```

## License

(C) Copyright 2022 by EPC Consulting und Software GmbH.